/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.extended.panel;

import com.alee.extended.layout.ToolbarLayout;
import com.alee.laf.StyleConstants;
import com.alee.laf.button.WebButton;
import com.alee.laf.label.WebLabel;
import com.alee.laf.panel.WebPanel;
import com.alee.utils.LafUtils;
import com.alee.utils.laf.ShapeProvider;
import com.alee.utils.swing.Timer;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

/**
 * User: mgarin Date: 09.11.11 Time: 14:38
 */

public class WebCollapsiblePane extends WebPanel implements ShapeProvider
{
    public static final ImageIcon expand =
            new ImageIcon ( WebCollapsiblePane.class.getResource ( "icons/expand.png" ) );
    public static final ImageIcon collapse =
            new ImageIcon ( WebCollapsiblePane.class.getResource ( "icons/collapse.png" ) );

    private boolean animate = StyleConstants.animate;
    private boolean expanded = true;
    private float expandState = 1f;
    private float expandSpeed = 0.1f;
    private Timer animator = null;

    private Component content = null;

    private WebPanel headerPanel;
    private WebLabel titleLabel;
    private WebButton expandButton;

    private WebPanel contentPanel;

    public WebCollapsiblePane ()
    {
        this ( "" );
    }

    public WebCollapsiblePane ( String title )
    {
        this ( title, ( ImageIcon ) null );
    }

    public WebCollapsiblePane ( String title, ImageIcon icon )
    {
        this ( title, icon, null );
    }

    public WebCollapsiblePane ( String title, Component content )
    {
        this ( title, null, content );
    }

    public WebCollapsiblePane ( String title, ImageIcon icon, Component content )
    {
        super ();
        setUndecorated ( false );
        setRound ( StyleConstants.smallRound );

        this.content = content;

        setLayout ( new BorderLayout ( 0, 0 ) );

        add ( createHeaderPanel ( title, icon ), BorderLayout.NORTH );

        contentPanel = new WebPanel ()
        {
            public Dimension getPreferredSize ()
            {
                Dimension ps = super.getPreferredSize ();
                if ( expandState < 1f )
                {
                    ps.height = Math.round ( ps.height * expandState );
                }
                return ps;
            }
        };
        contentPanel.setOpaque ( false );
        contentPanel.setLayout ( new BorderLayout ( 0, 0 ) );
        add ( contentPanel, BorderLayout.CENTER );

        if ( content != null )
        {
            contentPanel.add ( content, BorderLayout.CENTER );
        }
    }

    private WebPanel createHeaderPanel ( String title, ImageIcon icon )
    {
        headerPanel = new WebPanel ();

        headerPanel.setOpaque ( true );
        headerPanel.setUndecorated ( false );
        headerPanel.setDrawSides ( false, false, true, false );
        headerPanel.setShadeWidth ( 0 );

        headerPanel.setLayout ( new ToolbarLayout ( 0 ) );
        headerPanel.addMouseListener ( new MouseAdapter ()
        {
            public void mouseReleased ( MouseEvent e )
            {
                if ( SwingUtilities.isLeftMouseButton ( e ) && e.getX () >= 0 && e.getY () >= 0 &&
                        e.getX () <= getWidth () && e.getY () <= getHeight () )
                {
                    invertExpandState ();
                }
            }
        } );

        titleLabel = new WebLabel ( title, icon, WebLabel.LEFT );
        updateTitleBorder ();
        headerPanel.add ( titleLabel, ToolbarLayout.START );

        expandButton = WebButton.createIconWebButton ( collapse, StyleConstants.smallRound, true );
        expandButton.setDrawFocus ( false );
        expandButton.setFocusable ( true );
        expandButton.setInnerShadeWidth ( 0 );
        expandButton.addActionListener ( new ActionListener ()
        {
            public void actionPerformed ( ActionEvent e )
            {
                invertExpandState ();
            }
        } );
        headerPanel.add ( expandButton, ToolbarLayout.END );

        return headerPanel;
    }

    private void updateTitleBorder ()
    {
        if ( titleLabel != null )
        {
            if ( titleLabel.getIcon () == null )
            {
                titleLabel.setBorder ( BorderFactory.createEmptyBorder ( 2, 4, 2, 0 ) );
            }
            else
            {
                titleLabel.setBorder ( BorderFactory.createEmptyBorder ( 2, 2, 2, 0 ) );
            }
        }
    }

    public void invertExpandState ()
    {
        invertExpandState ( animate );
    }

    public void invertExpandState ( boolean animate )
    {
        setExpanded ( !isExpanded (), animate );
    }

    public boolean isExpanded ()
    {
        return expanded;
    }

    public void setExpanded ( boolean expanded )
    {
        setExpanded ( expanded, animate );
    }

    public void setExpanded ( boolean expanded, boolean animate )
    {
        if ( expanded )
        {
            expand ( animate );
        }
        else
        {
            collapse ( animate );
        }
    }

    public void collapse ()
    {
        collapse ( animate );
    }

    public void collapse ( boolean animate )
    {
        if ( !expanded )
        {
            return;
        }

        stopAnimation ();

        expanded = false;
        expandButton.setIcon ( expand );

        if ( animate )
        {
            animator = new Timer ( 1000 / StyleConstants.averageAnimationFps, null );
            animator.addActionListener ( new ActionListener ()
            {
                public void actionPerformed ( ActionEvent e )
                {
                    if ( expandState > 0f )
                    {
                        expandState = Math.max ( 0, expandState - expandSpeed );
                        WebCollapsiblePane.this.revalidate ();
                    }
                    else
                    {
                        headerPanel.setDrawBottom ( false );
                        WebCollapsiblePane.this.revalidate ();
                        WebCollapsiblePane.this.repaint ();
                        animator.stop ();
                    }
                }
            } );
            animator.start ();
        }
        else
        {
            expandState = 0f;
            headerPanel.setDrawBottom ( false );
            WebCollapsiblePane.this.revalidate ();
            WebCollapsiblePane.this.repaint ();
        }
    }

    public void expand ()
    {
        expand ( animate );
    }

    public void expand ( boolean animate )
    {
        if ( expanded )
        {
            return;
        }

        stopAnimation ();

        expanded = true;
        expandButton.setIcon ( collapse );

        headerPanel.setDrawBottom ( true );

        if ( animate )
        {
            animator = new Timer ( 1000 / StyleConstants.averageAnimationFps, null );
            animator.addActionListener ( new ActionListener ()
            {
                public void actionPerformed ( ActionEvent e )
                {
                    if ( expandState < 1f )
                    {
                        expandState = Math.min ( 1f, expandState + expandSpeed );
                        WebCollapsiblePane.this.revalidate ();
                    }
                    else
                    {
                        WebCollapsiblePane.this.revalidate ();
                        WebCollapsiblePane.this.repaint ();
                        animator.stop ();
                    }
                }
            } );
            animator.start ();
        }
        else
        {
            expandState = 1f;
            WebCollapsiblePane.this.revalidate ();
            WebCollapsiblePane.this.repaint ();
        }
    }

    private void stopAnimation ()
    {
        if ( animator != null && animator.isRunning () )
        {
            animator.stop ();
        }
    }

    public boolean isAnimate ()
    {
        return animate;
    }

    public void setAnimate ( boolean animate )
    {
        this.animate = animate;
    }

    public void setTitle ( String title )
    {
        titleLabel.setText ( title );
    }

    public String getTitle ()
    {
        return titleLabel.getText ();
    }

    public void setIcon ( Icon icon )
    {
        titleLabel.setIcon ( icon );
        updateTitleBorder ();
    }

    public Icon getIcon ()
    {
        return titleLabel.getIcon ();
    }

    public WebPanel getHeaderPanel ()
    {
        return headerPanel;
    }

    public WebLabel getTitleLabel ()
    {
        return titleLabel;
    }

    public WebButton getExpandButton ()
    {
        return expandButton;
    }

    public Component getContent ()
    {
        return content;
    }

    public void setContent ( Component content )
    {
        if ( this.content != null )
        {
            contentPanel.remove ( content );
        }

        this.content = content;
        contentPanel.add ( content, BorderLayout.CENTER );
        revalidate ();
    }

    public Shape provideShape ()
    {
        return LafUtils
                .getWebBorderShape ( WebCollapsiblePane.this, getShadeWidth (), getRound () );
    }
}
