/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.laf.panel;

import com.alee.extended.background.BackgroundPainter;
import com.alee.laf.StyleConstants;
import com.alee.utils.LafUtils;
import com.alee.utils.SwingUtils;
import com.alee.utils.laf.ShapeProvider;

import javax.swing.*;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicPanelUI;
import java.awt.*;
import java.awt.geom.GeneralPath;

/**
 * User: mgarin Date: 26.07.11 Time: 13:11
 */

public class WebPanelUI extends BasicPanelUI implements ShapeProvider
{
    private boolean undecorated = WebPanelStyle.undecorated;
    private int round = WebPanelStyle.round;
    private int shadeWidth = WebPanelStyle.shadeWidth;
    private Insets margin = WebPanelStyle.margin;
    private boolean fillBackground = WebPanelStyle.fillBackground;
    private boolean webColored = WebPanelStyle.webColored;
    private BackgroundPainter backgroundPainter = WebPanelStyle.backgroundPainter;

    private boolean drawTop = WebPanelStyle.drawTop;
    private boolean drawLeft = WebPanelStyle.drawLeft;
    private boolean drawBottom = WebPanelStyle.drawBottom;
    private boolean drawRight = WebPanelStyle.drawRight;

    private JPanel panel = null;

    public static ComponentUI createUI ( JComponent c )
    {
        return new WebPanelUI ();
    }

    public void installUI ( JComponent c )
    {
        super.installUI ( c );

        panel = ( JPanel ) c;

        c.setOpaque ( true );
        c.setBackground ( WebPanelStyle.backgroundColor );

        updateBorder ( c );
    }

    public Shape provideShape ()
    {
        return LafUtils.getWebBorderShape ( panel, getShadeWidth (), getRound () );
    }

    private void updateBorder ( JComponent component )
    {
        if ( backgroundPainter != null )
        {
            // Background insets
            Insets bi = backgroundPainter.getBackgroundMargin ( component );
            component.setBorder ( BorderFactory
                    .createEmptyBorder ( margin.top + bi.top, margin.left + bi.left,
                            margin.bottom + bi.bottom, margin.right + bi.right ) );
        }
        else if ( !undecorated )
        {
            // Panel insets
            int top = margin.top + ( drawTop ? shadeWidth + 1 : 0 );
            int left = margin.left + ( drawLeft ? shadeWidth + 1 : 0 );
            int bottom = margin.bottom + ( drawBottom ? shadeWidth + 1 : 0 );
            int right = margin.right + ( drawRight ? shadeWidth + 1 : 0 );
            component.setBorder ( BorderFactory.createEmptyBorder ( top, left, bottom, right ) );
        }
        else
        {
            // Empty insets
            component.setBorder ( BorderFactory
                    .createEmptyBorder ( margin.top, margin.left, margin.bottom, margin.right ) );
        }
    }

    public boolean isUndecorated ()
    {
        return undecorated;
    }

    public void setUndecorated ( boolean undecorated )
    {
        this.undecorated = undecorated;

        // Updating border
        updateBorder ( panel );

        // Updating opaque value
        if ( !undecorated )
        {
            panel.setOpaque ( false );
        }
    }

    public BackgroundPainter getBackgroundPainter ()
    {
        return backgroundPainter;
    }

    public void setBackgroundPainter ( BackgroundPainter backgroundPainter )
    {
        this.backgroundPainter = backgroundPainter;
        updateBorder ( panel );
    }

    public int getRound ()
    {
        if ( undecorated )
        {
            return 0;
        }
        else
        {
            return round;
        }
    }

    public void setRound ( int round )
    {
        this.round = round;
    }

    public int getShadeWidth ()
    {
        if ( undecorated )
        {
            return 0;
        }
        else
        {
            return shadeWidth;
        }
    }

    public void setShadeWidth ( int shadeWidth )
    {
        this.shadeWidth = shadeWidth;
        updateBorder ( panel );
    }

    public Insets getMargin ()
    {
        return margin;
    }

    public void setMargin ( Insets margin )
    {
        this.margin = margin;
        updateBorder ( panel );
    }

    public boolean isFillBackground ()
    {
        return fillBackground;
    }

    public void setFillBackground ( boolean fillBackground )
    {
        this.fillBackground = fillBackground;
    }

    public boolean isWebColored ()
    {
        return webColored;
    }

    public void setWebColored ( boolean webColored )
    {
        this.webColored = webColored;
    }

    public boolean isDrawBottom ()
    {
        return drawBottom;
    }

    public void setDrawBottom ( boolean drawBottom )
    {
        this.drawBottom = drawBottom;
        updateBorder ( panel );
    }

    public boolean isDrawLeft ()
    {
        return drawLeft;
    }

    public void setDrawLeft ( boolean drawLeft )
    {
        this.drawLeft = drawLeft;
        updateBorder ( panel );
    }

    public boolean isDrawRight ()
    {
        return drawRight;
    }

    public void setDrawRight ( boolean drawRight )
    {
        this.drawRight = drawRight;
        updateBorder ( panel );
    }

    public boolean isDrawTop ()
    {
        return drawTop;
    }

    public void setDrawTop ( boolean drawTop )
    {
        this.drawTop = drawTop;
        updateBorder ( panel );
    }

    public void setDrawSides ( boolean top, boolean left, boolean bottom, boolean right )
    {
        this.drawTop = top;
        this.drawLeft = left;
        this.drawBottom = bottom;
        this.drawRight = right;
        updateBorder ( panel );
    }

    public void paint ( Graphics g, JComponent c )
    {
        if ( backgroundPainter != null )
        {
            // Use background painter instead of default UI graphics
            backgroundPainter.paintBackground ( g, SwingUtils.size ( c ), c );
        }
        else if ( !undecorated )
        {
            // Checking need of painting
            boolean anyBorder = drawTop || drawRight || drawBottom || drawLeft;
            if ( anyBorder || fillBackground )
            {
                Graphics2D g2d = ( Graphics2D ) g;
                Object aa = LafUtils.setupAntialias ( g2d );

                // Border shape
                Shape borderShape = getBorderShape ( c );
                Insets insets = c.getInsets ();
                Rectangle bgShape = getBackgroundShape ( c );

                // Outer shadow
                if ( anyBorder )
                {
                    if ( c.isEnabled () )
                    {
                        LafUtils.drawShade ( g2d, borderShape, StyleConstants.shadeColor,
                                shadeWidth );
                    }
                }

                // Background
                if ( fillBackground )
                {
                    if ( webColored )
                    {
                        // Setup cached gradient paint
                        g2d.setPaint ( LafUtils.getWebBorderGradientPaint ( 0, bgShape.y, 0,
                                bgShape.y + bgShape.height ) );
                        g2d.fill ( bgShape );
                    }
                    else
                    {
                        // Setup single color paint
                        g2d.setPaint ( c.getBackground () );
                        g2d.fill ( bgShape );
                    }
                }

                // Border
                if ( anyBorder )
                {
                    g2d.setPaint ( c.isEnabled () ? StyleConstants.darkBorderColor :
                            StyleConstants.disabledBorderColor );
                    g2d.draw ( borderShape );
                }

                LafUtils.restoreAntialias ( g2d, aa );
            }
        }
    }

    private Rectangle getBackgroundShape ( JComponent c )
    {
        Insets insets = c.getInsets ();
        return new Rectangle ( drawLeft ? shadeWidth : 0, drawTop ? shadeWidth : 0,
                c.getWidth () - ( drawLeft ? shadeWidth : 0 ) - ( drawRight ? shadeWidth : 0 ),
                c.getHeight () - ( drawTop ? shadeWidth : 0 ) - ( drawBottom ? shadeWidth : 0 ) );
    }

    private Shape getBorderShape ( JComponent c )
    {
        GeneralPath shape = new GeneralPath ( GeneralPath.WIND_EVEN_ODD );
        int w = c.getWidth ();
        int h = c.getHeight ();
        if ( drawTop )
        {
            shape.moveTo ( drawLeft ? shadeWidth + round : 0, shadeWidth );
            if ( drawRight )
            {
                shape.lineTo ( w - shadeWidth - round - 1, shadeWidth );
                shape.quadTo ( w - shadeWidth - 1, shadeWidth, w - shadeWidth - 1,
                        shadeWidth + round );
            }
            else
            {
                shape.lineTo ( w - 1, shadeWidth );
            }
        }
        if ( drawRight )
        {
            shape.moveTo ( w - shadeWidth - 1, drawTop ? shadeWidth + round : 0 );
            if ( drawBottom )
            {
                shape.lineTo ( w - shadeWidth - 1, h - shadeWidth - round - 1 );
                shape.quadTo ( w - shadeWidth - 1, h - shadeWidth - 1, w - shadeWidth - round - 1,
                        h - shadeWidth - 1 );
            }
            else
            {
                shape.lineTo ( w - shadeWidth - 1, h - 1 );
            }
        }
        if ( drawBottom )
        {
            shape.moveTo ( drawRight ? w - shadeWidth - round - 1 : w - 1, h - shadeWidth - 1 );
            if ( drawLeft )
            {
                shape.lineTo ( shadeWidth + round, h - shadeWidth - 1 );
                shape.quadTo ( shadeWidth, h - shadeWidth - 1, shadeWidth,
                        h - shadeWidth - round - 1 );
            }
            else
            {
                shape.lineTo ( 0, h - shadeWidth - 1 );
            }
        }
        if ( drawLeft )
        {
            shape.moveTo ( shadeWidth, drawBottom ? h - shadeWidth - round - 1 : h - 1 );
            if ( drawTop )
            {
                shape.lineTo ( shadeWidth, shadeWidth + round );
                shape.quadTo ( shadeWidth, shadeWidth, shadeWidth + round, shadeWidth );
            }
            else
            {
                shape.lineTo ( shadeWidth, 0 );
            }
        }
        return shape;
    }

    public Dimension getPreferredSize ( JComponent c )
    {
        Dimension ps = c.getLayout () != null ? c.getLayout ().preferredLayoutSize ( c ) : null;
        if ( backgroundPainter != null )
        {
            ps = SwingUtils.max ( ps, backgroundPainter.getBackgroundPreferredSize ( c ) );
        }
        return ps;
    }
}
