/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.utils;

import com.alee.utils.file.FileDownloadListener;
import com.alee.utils.reflection.JarEntry;
import com.alee.utils.reflection.JarEntryType;
import com.alee.utils.reflection.JarStructure;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.security.CodeSource;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

/**
 * User: mgarin Date: 11/18/11 Time: 4:09 PM
 */

public class ReflectUtils
{
    /*
    * Retrieves class by canonical name safely
    */

    public static Class getClass ( String canonicalName )
    {
        try
        {
            return Class.forName ( canonicalName );
        }
        catch ( ClassNotFoundException e )
        {
            return null;
        }
    }

    /*
    * Creates jar structure from any class that is contained within that jar
    */

    public static JarStructure getJarStructure ( Class jarClass )
    {
        return getJarStructure ( jarClass, null, null );
    }

    public static JarStructure getJarStructure ( Class jarClass, List<String> allowedExtensions,
                                                 List<String> allowedPackgages )
    {
        return getJarStructure ( jarClass, allowedExtensions, allowedPackgages, null );
    }

    public static JarStructure getJarStructure ( Class jarClass, List<String> allowedExtensions,
                                                 List<String> allowedPackgages,
                                                 FileDownloadListener listener )
    {
        try
        {
            CodeSource src = jarClass.getProtectionDomain ().getCodeSource ();
            if ( src != null )
            {
                // Creating structure

                // Source url
                URL jarUrl = src.getLocation ();
                URI uri = jarUrl.toURI ();

                // Source file
                File jarFile;
                String scheme = uri.getScheme ();
                if ( scheme != null && scheme.equalsIgnoreCase ( "file" ) )
                {
                    // Local jar-file
                    jarFile = new File ( uri );
                }
                else
                {
                    // Remote jar-file
                    jarFile = FileUtils.downloadFile ( jarUrl.toString (),
                            File.createTempFile ( "jar_file", ".tmp" ), listener );
                }

                // Creating 
                JarEntry jarEntry = new JarEntry ( JarEntryType.jar, jarFile.getName () );
                JarStructure jarStructure = new JarStructure ( jarEntry );
                jarStructure.setJarLocation ( jarFile.getAbsolutePath () );

                // Reading all entries and parsing them into structure
                ZipInputStream zip = new ZipInputStream ( jarUrl.openStream () );
                ZipEntry zipEntry;
                while ( ( zipEntry = zip.getNextEntry () ) != null )
                {
                    String entryName = zipEntry.getName ();
                    if ( isAllowedPackage ( entryName, allowedPackgages ) &&
                            ( zipEntry.isDirectory () ||
                                    isAllowedExtension ( entryName, allowedExtensions ) ) )
                    {
                        parseElement ( jarEntry, entryName, zipEntry );
                    }
                }

                return jarStructure;
            }
        }
        catch ( IOException e )
        {
            e.printStackTrace ();
        }
        catch ( URISyntaxException e )
        {
            e.printStackTrace ();
        }
        return null;
    }

    private static boolean isAllowedExtension ( String entryName, List<String> allowedExtensions )
    {
        if ( allowedExtensions == null || allowedExtensions.size () == 0 )
        {
            return true;
        }
        else
        {
            String entryExt = FileUtils.getFileExtPart ( entryName, true ).toLowerCase ();
            return allowedExtensions.contains ( entryExt );
        }
    }

    private static boolean isAllowedPackage ( String entryName, List<String> allowedPackgages )
    {
        if ( allowedPackgages == null || allowedPackgages.size () == 0 )
        {
            return true;
        }
        else
        {
            for ( String packageStart : allowedPackgages )
            {
                if ( entryName.startsWith ( packageStart ) )
                {
                    return true;
                }
            }
            return false;
        }
    }

    private static void parseElement ( JarEntry jarEntry, String entryName, ZipEntry zipEntry )
    {
        String[] path = entryName.split ( "/" );
        JarEntry currentLevel = jarEntry;
        for ( int i = 0; i < path.length; i++ )
        {
            if ( i < path.length - 1 )
            {
                // We are getting deeper into packages
                JarEntry child = currentLevel.getChildByName ( path[ i ] );
                if ( child == null )
                {
                    child = new JarEntry ( JarEntryType.packageEntry, path[ i ], currentLevel );
                    child.setZipEntry ( zipEntry );
                    currentLevel.addChild ( child );
                }
                currentLevel = child;
            }
            else
            {
                // We reached last element
                JarEntry newEntry =
                        new JarEntry ( getJarEntryType ( path[ i ] ), path[ i ], currentLevel );
                newEntry.setZipEntry ( zipEntry );
                currentLevel.addChild ( newEntry );
            }
        }
    }

    private static JarEntryType getJarEntryType ( String file )
    {
        String ext = FileUtils.getFileExtPart ( file, false );
        if ( ext.equals ( "java" ) )
        {
            return JarEntryType.javaEntry;
        }
        else if ( ext.equals ( "class" ) )
        {
            return JarEntryType.classEntry;
        }
        else if ( !ext.isEmpty () )
        {
            return JarEntryType.fileEntry;
        }
        else
        {
            return JarEntryType.packageEntry;
        }
    }

    /*
    * Retrieves any method caller class
    */

    public static Class getCallerClass ()
    {
        try
        {
            // 0 - this method class
            // 1 - this method caller class
            // 2 - caller's class caller
            return Class.forName ( new Throwable ().getStackTrace ()[ 2 ].getClassName () );
        }
        catch ( Throwable e )
        {
            return null;
        }
    }

    /*
    * Retrieves class name with .java
    */

    public static String getJavaClassName ( Object classObject )
    {
        return getJavaClassName ( classObject.getClass () );
    }

    public static String getJavaClassName ( Class classType )
    {
        return getClassName ( classType ) + ".java";
    }

    /*
    * Retrieves class name only
    */

    public static String getClassName ( Object classObject )
    {
        return getClassName ( classObject.getClass () );
    }

    public static String getClassName ( Class classType )
    {
        String fullName = classType.getCanonicalName ();
        int dot = fullName.lastIndexOf ( "." );
        return dot != -1 ? fullName.substring ( dot + 1 ) : fullName;
    }

    /*
    * Retrieves class file name only
    */

    public static String getClassFileName ( Object classObject )
    {
        return getClassFileName ( classObject.getClass () );
    }

    public static String getClassFileName ( Class classType )
    {
        return ReflectUtils.getClassName ( classType ) + ".class";
    }

    /*
    * Retrieves class packages array
    */

    public static String[] getClassPackages ( Object classObject )
    {
        return getClassPackages ( classObject.getClass () );
    }

    public static String[] getClassPackages ( Class classType )
    {
        return getPackages ( classType.getPackage ().getName () );
    }

    public static String[] getPackages ( String packageName )
    {
        return packageName.split ( "\\." );
    }

    /*
    * Creates class instance by its canonical name and constructor arguments
    */

    public static Object createInstanceSafely ( String canonicalClassName, Object... arguments )
    {
        try
        {
            return createInstance ( canonicalClassName, arguments );
        }
        catch ( Throwable e )
        {
            return null;
        }
    }

    public static Object createInstance ( String canonicalClassName, Object... arguments )
            throws ClassNotFoundException, InvocationTargetException, IllegalAccessException,
            InstantiationException
    {
        Class theClass = ReflectUtils.class.getClassLoader ().loadClass ( canonicalClassName );
        Constructor constructor = theClass.getConstructors ()[ 0 ];
        return constructor.newInstance ( arguments );
    }
}
