/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.utils;

import com.alee.extended.background.NinePatchBackgroundPainter;
import com.alee.extended.background.StateBackgroundPainter;
import com.alee.extended.background.TextureBackgroundPainter;
import com.alee.managers.settings.SettingsGroup;
import com.alee.utils.image.NinePatchIcon;
import com.alee.utils.xml.ResourceFile;
import com.alee.utils.xml.ResourceList;
import com.alee.utils.xml.ResourceLocation;
import com.alee.utils.xml.ResourceMap;
import com.thoughtworks.xstream.XStream;

import javax.swing.*;
import java.awt.*;
import java.io.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

/**
 * User: mgarin Date: 17.02.12 Time: 12:01
 */

public class XmlUtils
{
    /**
     * Various resource files load methods
     */

    public static String loadString ( Object source )
    {
        return loadString ( loadResourceFile ( source ) );
    }

    public static String loadString ( ResourceFile resource )
    {
        if ( resource.getLocation ().equals ( ResourceLocation.url ) )
        {
            try
            {
                return FileUtils.readToString ( new BufferedReader ( new InputStreamReader (
                        new URL ( resource.getSource () ).openStream () ) ) );
            }
            catch ( IOException e )
            {
                e.printStackTrace ();
                return null;
            }
        }
        if ( resource.getLocation ().equals ( ResourceLocation.filePath ) )
        {
            return FileUtils.readToString ( new File ( resource.getSource () ) );
        }
        else if ( resource.getLocation ().equals ( ResourceLocation.nearClass ) )
        {
            try
            {
                return FileUtils.readToString ( Class.forName ( resource.getClassName () ),
                        resource.getSource () );
            }
            catch ( ClassNotFoundException e )
            {
                e.printStackTrace ();
                return null;
            }
        }
        else
        {
            return null;
        }
    }

    public static ImageIcon loadImageIcon ( Object source )
    {
        return loadImageIcon ( loadResourceFile ( source ) );
    }

    public static ImageIcon loadImageIcon ( ResourceFile resource )
    {
        if ( resource.getLocation ().equals ( ResourceLocation.url ) )
        {
            try
            {
                return new ImageIcon ( new URL ( resource.getSource () ) );
            }
            catch ( MalformedURLException e )
            {
                e.printStackTrace ();
                return null;
            }
        }
        if ( resource.getLocation ().equals ( ResourceLocation.filePath ) )
        {
            try
            {
                return new ImageIcon ( new File ( resource.getSource () ).getCanonicalPath () );
            }
            catch ( IOException e )
            {
                e.printStackTrace ();
                return null;
            }
        }
        else if ( resource.getLocation ().equals ( ResourceLocation.nearClass ) )
        {
            try
            {
                return new ImageIcon ( Class.forName ( resource.getClassName () )
                        .getResource ( resource.getSource () ) );
            }
            catch ( ClassNotFoundException e )
            {
                e.printStackTrace ();
                return null;
            }
        }
        else
        {
            return null;
        }
    }

    public static List<ImageIcon> loadImagesList ( Object source )
    {
        return loadImagesList ( loadResourceList ( source ) );
    }

    public static List<ImageIcon> loadImagesList ( ResourceList resourceList )
    {
        List<ImageIcon> icons = new ArrayList<ImageIcon> ();
        for ( ResourceFile resource : resourceList.getResources () )
        {
            ImageIcon imageIcon = loadImageIcon ( resource );
            if ( imageIcon != null )
            {
                icons.add ( imageIcon );
            }
        }
        return icons;
    }

    public static NinePatchIcon loadNinePatchIcon ( Object source )
    {
        return loadNinePatchIcon ( loadResourceFile ( source ) );
    }

    public static NinePatchIcon loadNinePatchIcon ( ResourceFile resource )
    {
        return new NinePatchIcon ( loadImageIcon ( resource ) );
    }

    public static StateBackgroundPainter loadStateBackgroundPainter ( Object source )
    {
        return loadStateBackgroundPainter ( loadResourceMap ( source ) );
    }

    public static StateBackgroundPainter loadStateBackgroundPainter ( ResourceMap resourceMap )
    {
        StateBackgroundPainter sbp = new StateBackgroundPainter ();
        for ( String key : resourceMap.getStates ().keySet () )
        {
            sbp.addStateIcon ( key, loadNinePatchIcon ( resourceMap.getState ( key ) ) );
        }
        return sbp;
    }

    public static NinePatchBackgroundPainter loadNinePatchBackgroundPainter ( Object source )
    {
        return loadNinePatchBackgroundPainter ( loadResourceFile ( source ) );
    }

    public static NinePatchBackgroundPainter loadNinePatchBackgroundPainter (
            ResourceFile resource )
    {
        return new NinePatchBackgroundPainter ( loadNinePatchIcon ( resource ) );
    }

    public static TextureBackgroundPainter loadTextureBackgroundPainter ( Object source )
    {
        return loadTextureBackgroundPainter ( loadResourceFile ( source ) );
    }

    public static TextureBackgroundPainter loadTextureBackgroundPainter ( ResourceFile resource )
    {
        return new TextureBackgroundPainter ( loadImageIcon ( resource ) );
    }

    /**
     * Basic resources load methods
     */

    public static ResourceMap loadResourceMap ( Object source )
    {
        return ( ResourceMap ) loadResource ( source );
    }

    public static ResourceList loadResourceList ( Object source )
    {
        return ( ResourceList ) loadResource ( source );
    }

    public static ResourceFile loadResourceFile ( Object source )
    {
        return ( ResourceFile ) loadResource ( source );
    }

    public static Object loadResource ( Object source )
    {
        if ( source instanceof URL )
        {
            return XmlUtils.fromXML ( ( URL ) source );
        }
        else if ( source instanceof String )
        {
            return XmlUtils.fromXML ( new File ( ( String ) source ) );
        }
        else if ( source instanceof File )
        {
            return XmlUtils.fromXML ( ( File ) source );
        }
        else if ( source instanceof Reader )
        {
            return XmlUtils.fromXML ( ( Reader ) source );
        }
        else if ( source instanceof InputStream )
        {
            return XmlUtils.fromXML ( ( InputStream ) source );
        }
        else
        {
            return null;
        }
    }

    /**
     * XStream xml serializer/deserializer instance
     */

    private static XStream xStream = null;

    public static XStream getXStream ()
    {
        if ( xStream == null )
        {
            initializeXStream ();
        }
        return xStream;
    }

    private static void initializeXStream ()
    {
        try
        {
            xStream = new XStream ();

            // SettingsManager aliases
            xStream.alias ( "SettingsGroup", SettingsGroup.class );

            // XML resources aliases
            xStream.alias ( "ResourceFile", ResourceFile.class );
            xStream.alias ( "ResourceLocation", ResourceLocation.class );
            xStream.alias ( "ResourceList", ResourceList.class );
            xStream.alias ( "ResourceMap", ResourceMap.class );

            // Standart Java-classes aliases
            xStream.alias ( "Point", Point.class );
            xStream.alias ( "Dimension", Dimension.class );
            xStream.alias ( "Rectangle", Rectangle.class );
            xStream.alias ( "Font", Font.class );
            xStream.alias ( "Color", Color.class );
        }
        catch ( Throwable e )
        {
            e.printStackTrace ();
        }
    }

    /**
     * Additional aliases adding
     */

    public static void alias ( String name, Class type )
    {
        getXStream ().alias ( name, type );
    }

    /**
     * XStream methods references
     */

    public static String toXml ( Object obj )
    {
        return getXStream ().toXML ( obj );
    }

    public static void toXML ( Object obj, Writer out )
    {
        getXStream ().toXML ( obj, out );
    }

    public static void toXML ( Object obj, OutputStream out )
    {
        getXStream ().toXML ( obj, out );
    }

    public static void toXML ( Object obj, File file )
    {
        try
        {
            toXML ( obj, new FileWriter ( file ) );
        }
        catch ( IOException e )
        {
            e.printStackTrace ();
        }
    }

    public static void toXML ( Object obj, String file )
    {
        toXML ( obj, new File ( file ) );
    }

    public static Object fromXML ( Object source )
    {
        return fromXML ( loadResourceFile ( source ) );
    }

    public static Object fromXML ( ResourceFile resource )
    {
        if ( resource.getLocation ().equals ( ResourceLocation.url ) )
        {
            try
            {
                return XmlUtils.fromXML ( new URL ( resource.getSource () ) );
            }
            catch ( MalformedURLException e )
            {
                e.printStackTrace ();
                return null;
            }
        }
        if ( resource.getLocation ().equals ( ResourceLocation.filePath ) )
        {
            return XmlUtils.fromXML ( new File ( resource.getSource () ) );
        }
        else if ( resource.getLocation ().equals ( ResourceLocation.nearClass ) )
        {
            try
            {
                return XmlUtils.fromXML ( Class.forName ( resource.getClassName () )
                        .getResourceAsStream ( resource.getSource () ) );
            }
            catch ( ClassNotFoundException e )
            {
                e.printStackTrace ();
                return null;
            }
        }
        else
        {
            return null;
        }
    }

    public static Object fromXML ( String xml )
    {
        return getXStream ().fromXML ( xml );
    }

    public static Object fromXML ( Reader reader )
    {
        return getXStream ().fromXML ( reader );
    }

    public static Object fromXML ( InputStream input )
    {
        return getXStream ().fromXML ( input );
    }

    public static Object fromXML ( URL url )
    {
        return getXStream ().fromXML ( url );
    }

    public static Object fromXML ( File file )
    {
        return getXStream ().fromXML ( file );
    }
}