/*
* This file is part of WebLookAndFeel library.
*
* WebLookAndFeel library is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* WebLookAndFeel library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with WebLookAndFeel library.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.alee.utils.swing;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

/**
 * User: mgarin Date: 14.03.12 Time: 17:48
 * <p/>
 * This class is a small replacement for standart javax.swing.Timer. Instead of running in a single
 * queue it creates separate Threads for each timer and does not affect event-dispatching thread,
 * until events are dispatched. This basically means that you can use any number of this Timer
 * instances and you can run them alltogether without having serious problems with
 */

public class Timer
{
    private int id = 0;
    private int lastId;
    private Map<Integer, Boolean> running = new Hashtable<Integer, Boolean> ();

    private List<ActionListener> listeners = new ArrayList<ActionListener> ();

    private long delay;
    private long initialDelay;
    private boolean repeats = true;
    private boolean coalesce = true;
    private boolean useEventDispatchThread = true;
    private String actionCommand = "";

    private Thread exec = null;

    public Timer ( long delay )
    {
        this ( delay, null );
    }

    public Timer ( long delay, long initialDelay )
    {
        this ( delay, initialDelay, null );
    }

    public Timer ( long delay, ActionListener listener )
    {
        this ( delay, delay, listener );
    }

    public Timer ( long delay, long initialDelay, ActionListener listener )
    {
        super ();
        setDelay ( delay );
        setInitialDelay ( initialDelay );
        addActionListener ( listener );
    }

    public long getInitialDelay ()
    {
        return initialDelay;
    }

    public void setInitialDelay ( long initialDelay )
    {
        if ( initialDelay < 0 )
        {
            throw new IllegalArgumentException ( "Invalid initial delay: " + initialDelay );
        }
        else
        {
            this.initialDelay = initialDelay;
        }
    }

    public long getDelay ()
    {
        return delay;
    }

    public void setDelay ( long delay )
    {
        if ( delay < 0 )
        {
            throw new IllegalArgumentException ( "Invalid delay: " + delay );
        }
        else
        {
            this.delay = delay;
        }
    }

    public boolean isRepeats ()
    {
        return repeats;
    }

    public void setRepeats ( boolean repeats )
    {
        this.repeats = repeats;
    }

    public boolean isCoalesce ()
    {
        return coalesce;
    }

    public void setCoalesce ( boolean coalesce )
    {
        this.coalesce = coalesce;
    }

    public boolean isUseEventDispatchThread ()
    {
        return useEventDispatchThread;
    }

    public void setUseEventDispatchThread ( boolean useEventDispatchThread )
    {
        this.useEventDispatchThread = useEventDispatchThread;
    }

    public String getActionCommand ()
    {
        return actionCommand;
    }

    public void setActionCommand ( String actionCommand )
    {
        this.actionCommand = actionCommand;
    }

    public void addActionListener ( ActionListener listener )
    {
        if ( listener != null )
        {
            listeners.add ( listener );
        }
    }

    public void removeActionListener ( ActionListener listener )
    {
        if ( listener != null )
        {
            listeners.remove ( listener );
        }
    }

    public List<ActionListener> getListeners ()
    {
        return listeners;
    }

    public boolean isRunning ()
    {
        return running.values ().contains ( true );
    }

    public void start ()
    {
        startExec ();
    }

    public void stop ()
    {
        stopExec ();
    }

    public void restart ()
    {
        stopExec ();
        startExec ();
    }

    private synchronized void startExec ()
    {
        // Ignore if timer is already running
        if ( isRunning () )
        {
            return;
        }

        // Saving current thread unique id
        lastId = id;
        id++;

        // Starting new cycling thread
        final int currentId = lastId;
        exec = new Thread ( new Runnable ()
        {
            public void run ()
            {
                // Adding a live thread into map
                running.put ( currentId, true );

                try
                {
                    // Initial delay
                    Thread.sleep ( initialDelay );
                    if ( repeats )
                    {
                        // Repeated events
                        // noinspection InfiniteLoopStatement
                        while ( shouldContinue () )
                        {
                            // Firing events
                            fireEvent ();

                            // Checking if we sould stop execution due to changes through events
                            if ( !shouldContinue () )
                            {
                                break;
                            }

                            // Waiting for next execution
                            Thread.sleep ( delay );
                        }
                    }
                    else
                    {
                        // Single event
                        fireEvent ();
                    }
                }
                catch ( InterruptedException e )
                {
                    //
                }

                // Removing finished thread from map
                running.remove ( currentId );
            }

            private boolean shouldContinue ()
            {
                return running.get ( currentId ) && !Thread.currentThread ().isInterrupted ();
            }
        } );
        exec.start ();
    }

    private synchronized void stopExec ()
    {
        // Interrupt thread if it is sleeping
        if ( exec != null )
        {
            exec.interrupt ();
            running.put ( lastId, false );
        }
    }

    private void fireEvent ()
    {
        if ( listeners.size () > 0 )
        {
            // Event
            final ActionEvent e =
                    new ActionEvent ( Timer.this, 0, actionCommand, System.currentTimeMillis (),
                            0 );

            // Working with local array
            final List<ActionListener> listenerList = new ArrayList<ActionListener> ();
            listenerList.addAll ( listeners );

            // Dispatch event in chosen way
            if ( useEventDispatchThread )
            {
                if ( coalesce )
                {
                    // Merge all events into single call to event dispatch thread
                    SwingUtilities.invokeLater ( new Runnable ()
                    {
                        public void run ()
                        {
                            for ( ActionListener listener : listenerList )
                            {
                                listener.actionPerformed ( e );
                            }
                        }
                    } );
                }
                else
                {
                    // Make separate event calls to event dispatch thread
                    for ( final ActionListener listener : listenerList )
                    {
                        SwingUtilities.invokeLater ( new Runnable ()
                        {
                            public void run ()
                            {
                                listener.actionPerformed ( e );
                            }
                        } );
                    }
                }
            }
            else
            {
                // Execute events in the same thread with timer
                for ( ActionListener listener : listenerList )
                {
                    listener.actionPerformed ( e );
                }
            }
        }
    }
}